@@ -81,16 +81,8 @@ enum {
 };
 
 static gboolean cards_are_droppable (hslot_type hslot) {
-      SCM cardlist = SCM_EOL;
-      GList* temp;
-
-      if (!hslot) return FALSE;
-
-      for (temp = press_data->cards; temp; temp = temp->next)
-        cardlist = scm_cons(make_card(temp->data), cardlist);
-      return SCM_NFALSEP (cscmi_droppable_lambda (scm_long2num (press_data->hslot->id),
-                                                  cardlist,
-                                                  scm_long2num (hslot->id)));
+  return hslot && press_data->hslot &&
+    cscmi_drop_valid(press_data->hslot->id, press_data->cards, hslot->id);
 }
 
 static hslot_type find_drop_target(gint x, gint y) {
@@ -168,13 +160,13 @@ int waiting_for_mouse_up(void) {
 }
 
 void end_of_game_test() {
-  scm_c_eval_string ("(end-move)");
-  if(!game_over) { 
-    game_over = !SCM_NFALSEP (cscmi_game_over_lambda ());
+  cscmi_end_move();
+  if (!game_over) { 
+    game_over = !cscmi_game_over_lambda();
     
     if (game_over) {
       timer_stop ();
-      game_won = SCM_NFALSEP (cscmi_winning_game_lambda ());
+      game_won = cscmi_winning_game_lambda();
       refresh_screen ();
       if (game_won) { /* Check because we can undo lost games. */
 	game_in_progress = FALSE;
@@ -187,40 +179,27 @@ void end_of_game_test() {
 }
 
 void drop_moving_cards(gint x, gint y) {
-  GList* temp;
-  SCM cardlist = SCM_EOL;
   hslot_type hslot;
-  gint moved = 0;
-  gint width, height;
+  gboolean moved = FALSE;
 
   hslot = find_drop_target(x - press_data->xoffset, y - press_data->yoffset);
 
   if (hslot) {
-    for (temp = press_data->cards; temp; temp = temp->next)
-      cardlist = scm_cons(make_card(temp->data), cardlist);
-    moved = SCM_NFALSEP (cscmi_button_released_lambda (scm_long2num (press_data->hslot->id),
-						       cardlist, 
-						       scm_long2num (hslot->id)));
+    moved = cscmi_drop_cards(press_data->hslot->id, press_data->cards, 
+			     hslot->id);
   }
 
   if (!moved) {
-    hslot = press_data->hslot;
-    add_cards_to_slot(press_data->cards, hslot);
-    scm_c_eval_string ("(discard-move)");
+    add_cards_to_slot(press_data->cards, press_data->hslot);
+    press_data->cards = NULL; /* they belong to the slot now */
+    cscmi_discard_move();
   }
 
-  update_slot_length(hslot);
-
-  gdk_drawable_get_size(press_data->moving_cards, &width, &height);
-  gdk_window_move(press_data->moving_cards, 
-		  hslot->pixelx + hslot->width - width, 
-		  hslot->pixely + hslot->height - height);
-
   refresh_screen();
 
-  free_press_data ();
+  free_press_data();
 
-  if(moved) end_of_game_test();
+  if (moved) end_of_game_test();
 }
 
 static void highlight_drop_target(hslot_type hslot) {
@@ -233,8 +212,7 @@ static void highlight_drop_target(hslot_type hslot) {
     if (cards_are_droppable(hslot)) {
       GdkPixmap *droptarget_pixmap;
 
-      if (hslot->length) {
-      
+      if (hslot->cards) {      
         hcard_type card;
 
         gint delta = hslot->exposed ? hslot->exposed - 1 : 0;
@@ -242,7 +220,7 @@ static void highlight_drop_target(hslot_type hslot) {
         int x = hslot->pixelx + delta * hslot->pixeldx;
         int y = hslot->pixely + delta * hslot->pixeldy;
 
-        card = g_list_nth(hslot->cards, hslot->length - 1)->data;
+        card = g_list_last(hslot->cards)->data;
 
         droptarget_pixmap = get_droptarget_pixmap (card->suit, 
 						   card->value, 
@@ -252,7 +230,7 @@ static void highlight_drop_target(hslot_type hslot) {
           gdk_window_set_back_pixmap (press_data->highlight_window,
                                       droptarget_pixmap, 0);
           gdk_window_shape_combine_mask (press_data->highlight_window,
-                                    	 mask, 0, 0);
+                                    	 card_mask, 0, 0);
           gdk_window_resize(press_data->highlight_window, 
 			    card_width, card_height);
           gdk_window_clear(press_data->highlight_window);
@@ -262,8 +240,8 @@ static void highlight_drop_target(hslot_type hslot) {
       } 
     }
 
-  /* This ensures that the moving cards are on top.*/
-  gdk_window_show (press_data->moving_cards);
+    /* This ensures that the moving cards are on top.*/
+    gdk_window_show (press_data->moving_cards); 
   }
 }
 
@@ -276,25 +254,33 @@ gint button_press_event (GtkWidget *widget, GdkEventButton *event, void *d)
   gboolean double_click;
   gboolean drag_valid;
 
+  double_click = FALSE;
+
   if (!game_in_progress) {
     game_in_progress = TRUE;
     option_list_set_sensitive ();
     timer_start ();
   }
 
-  if (event->button != 1 && event->button != 3)
+  if (event->button < 1 && event->button > 3)
     return TRUE;
   
   /* ignore the gdk synthetic click events */
   if (event->type != GDK_BUTTON_PRESS)
     return TRUE;
   
+  /* Let the middle button be a synthetic double-click. */
+  if (event->button == 2) {
+    double_click = TRUE;
+    event->button = 1;
+  }
+
   if (press_data->status == STATUS_SHOW || press_data->status == STATUS_IS_DRAG)
     return TRUE;
 
   slot_pressed(event->x, event->y, &hslot, &cardid);
 
-  if (!hslot)
+  if (!hslot) 
     return TRUE;
 
   if (event->button == 1)
@@ -303,10 +289,10 @@ gint button_press_event (GtkWidget *widget, GdkEventButton *event, void *d)
   /* We can't let Gdk do the double-click detection since the entire playing
    * area is one big widget it can't distinguish between single-clicks on two
    * cards and a double-click on one. */
-  double_click = (g_timer_elapsed(click_timer,NULL) < dbl_click_time)
+  double_click |= (g_timer_elapsed(click_timer, NULL) < dbl_click_time)
     && (last_cardid == cardid)
     && (last_hslot->id == hslot->id)
-    && (event->button != 3);
+    && (event->button == 1);
   g_timer_start(click_timer);
   slot_pressed(event->x, event->y, &last_hslot, &last_cardid);
 
@@ -320,14 +306,14 @@ gint button_press_event (GtkWidget *widget, GdkEventButton *event, void *d)
       && event->button == 3) {
     hcard_type card = g_list_nth(hslot->cards, cardid - 1)->data;
 
-    if (card->direction == UP) {      
+    if (card->direction == UP) {
       guint delta = hslot->exposed - (hslot->length - cardid) - 1;
       int x = hslot->pixelx + delta * hslot->pixeldx;
       int y = hslot->pixely + delta * hslot->pixeldy;
 
       press_data->status = STATUS_SHOW;
       press_data->moving_pixmap = get_card_picture (card->suit, card->value);
-      press_data->moving_mask = mask;
+      press_data->moving_mask = card_mask;
 
       if (press_data->moving_pixmap != NULL)
         gdk_window_set_back_pixmap (press_data->moving_cards, 
@@ -347,15 +333,8 @@ gint button_press_event (GtkWidget *widget, GdkEventButton *event, void *d)
 
   /* Check if the cards are draggable, assuming we have any cards. */
   if (hslot->cards) {
-    hslot_type hslot = press_data->hslot;
-    GList* glist = g_list_nth(hslot->cards, press_data->cardid - 1); 
-    SCM cardlist = SCM_EOL;
-
-    for (; glist; glist = glist->next)
-      cardlist = scm_cons(make_card((hcard_type)glist->data), cardlist);
-    
-    drag_valid = SCM_NFALSEP (cscmi_button_pressed_lambda (scm_long2num (press_data->hslot->id), 
-							   cardlist));
+    GList* cards = g_list_nth(hslot->cards, press_data->cardid - 1); 
+    drag_valid = cscmi_drag_valid(press_data->hslot->id, cards);
   } else {
     drag_valid = FALSE;
   }
@@ -371,12 +350,11 @@ gint button_press_event (GtkWidget *widget, GdkEventButton *event, void *d)
 
   if (double_click) {
     press_data->status = STATUS_NONE; 
-    scm_call_2 (scm_c_eval_string ("record-move"), scm_long2num (-1),
-	      SCM_EOL);
-    if (SCM_NFALSEP (cscmi_button_double_clicked_lambda (scm_long2num (hslot->id))))
-      scm_call_0 (scm_c_eval_string ("end-move"));
-    else {
-      scm_call_0 (scm_c_eval_string ("discard-move"));
+    cscmi_record_move(-1, NULL);
+    if (cscmi_button_double_clicked_lambda (hslot->id)) {
+      cscmi_end_move();
+    } else {
+      cscmi_discard_move();
       /* Allow for a drag on the second click if nothing else happened. */
       if ((cardid > 0) && drag_valid) {
 	press_data->status = STATUS_MAYBE_DRAG; 
@@ -411,6 +389,7 @@ gint button_release_event (GtkWidget *widget, GdkEventButton *event, void *d)
 
   if (event->button != 1)
     return TRUE;
+
   switch (press_data->status) {
   case STATUS_IS_DRAG:
     press_data->status = STATUS_NONE;
@@ -421,12 +400,11 @@ gint button_release_event (GtkWidget *widget, GdkEventButton *event, void *d)
   case STATUS_MAYBE_DRAG:
   case STATUS_NOT_DRAG:
     press_data->status = STATUS_CLICK;
-    scm_call_2 (scm_c_eval_string ("record-move"), scm_long2num (-1),
-	      SCM_EOL);
-    if (SCM_NFALSEP (cscmi_button_clicked_lambda (scm_long2num (press_data->hslot->id))))
-	    scm_call_0 (scm_c_eval_string ("end-move"));
+    cscmi_record_move(-1, NULL);
+    if (cscmi_button_clicked_lambda (press_data->hslot->id))
+      cscmi_end_move();
     else
-	    scm_call_0 (scm_c_eval_string ("discard-move"));
+      cscmi_discard_move();
     refresh_screen();
     end_of_game_test();
     break;
@@ -441,31 +419,49 @@ gint button_release_event (GtkWidget *widget, GdkEventButton *event, void *d)
 
 gint motion_notify_event (GtkWidget *widget, GdkEventMotion *event)
 {
+  gint x, y;
+  gint dummy;
+  hslot_type target_hslot;
+
+  x = event->x - press_data->xoffset;
+  y = event->y - press_data->yoffset;
+
   if (press_data->status == STATUS_IS_DRAG) {
-    hslot_type hslot;
+    target_hslot = find_drop_target(x, y);
+    highlight_drop_target(target_hslot);
 
-    hslot = find_drop_target(event->x - press_data->xoffset,
-			     event->y - press_data->yoffset);
-    
-    highlight_drop_target(hslot);
+    gdk_window_move(press_data->moving_cards, x, y);
 
-    gdk_window_move(press_data->moving_cards,  
-		    event->x - press_data->xoffset,
-		    event->y - press_data->yoffset);
+    /* This is a quick and nasty hack to make sure that the areas
+     * under the transparent corners of the dragged card get drawn
+     * properly. */
+    gdk_draw_drawable (playing_area->window, playing_area->style->black_gc,
+		       surface, x, y, x, y, 
+		       press_data->width, press_data->height);
 
-    gdk_window_clear(press_data->moving_cards);
+    gdk_window_clear(press_data->moving_cards); 
     set_cursor (CURSOR_CLOSED);
   }
   else if (press_data->status == STATUS_MAYBE_DRAG &&
-	   (abs(press_data->xoffset - event->x) > 2 ||
-	    abs(press_data->yoffset - event->y) > 2)) {
+	   (abs(x) > 2 || abs(y) > 2)) {
     press_data->status = STATUS_IS_DRAG;
     generate_press_data ();
     refresh_screen ();
-    take_snapshot();
     set_cursor (CURSOR_CLOSED);
   } else {
-    set_cursor_by_location (CURSOR_OPEN, event->x, event->y);
+    /* If we've moved off the card, abandon the click. */
+    slot_pressed (event->x, event->y, &target_hslot, &dummy);
+    if (target_hslot != press_data->hslot) {
+      if (press_data->status == STATUS_NOT_DRAG) {
+	press_data->status = STATUS_CLICK;
+      } 
+    } else {
+      if (press_data->status == STATUS_CLICK) {
+	press_data->status = STATUS_NOT_DRAG;
+      }
+    }
+    set_cursor_by_location (target_hslot == press_data->hslot ?
+			    CURSOR_CLOSED : CURSOR_OPEN, event->x, event->y);
   }
   return FALSE;
 }
@@ -526,6 +522,7 @@ gint configure_event (GtkWidget *widget, GdkEventConfigure *event) {
   surface = gdk_pixmap_new (playing_area->window, window_width, window_height, -1);
 
   rescale_cards ();
+
   refresh_screen();
 
   /* Set up the double-click detection.*/
@@ -535,6 +532,21 @@ gint configure_event (GtkWidget *widget, GdkEventConfigure *event) {
   g_object_get(G_OBJECT(settings),"gtk-double-click-time",&tmptime,NULL);
   dbl_click_time = tmptime/1000.0;
 
+
+  return FALSE;
+}
+
+gint expose_event (GtkWidget *widget, GdkEventExpose *event) 
+{
+
+  if (!surface)
+    return TRUE;
+
+  gdk_draw_drawable (widget->window, widget->style->black_gc,
+		     surface, event->area.x, event->area.y,
+		     event->area.x, event->area.y,
+		     event->area.width, event->area.height);
+
   return FALSE;
 }
 
