@@ -33,7 +33,6 @@
 #include "draw.h"
 #include <gnome.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-window-icon.h>
 #include <glade/glade.h>
 #include "slot.h"
 #include "card.h"
@@ -69,14 +68,12 @@ gboolean         dont_save = FALSE; /* If the game is selected on the
                                      * This is essential for Freecell
                                      * emulation.*/
 
-gint32		 enabled_features;
 gboolean	 droppable_is_featured;
 gboolean	 score_is_hidden; 
 
 guint            score;
 guint            timeout;
 guint32          seed;
-guint            n_games;
 gchar            *game_file = "";
 gchar            *game_name;
 gboolean         game_in_progress = FALSE;
@@ -86,10 +83,6 @@ gboolean         click_to_move = FALSE;
 gchar            *variation = "";
 gchar            *gamesdir;
 
-guint            x_spacing = 5;
-guint            y_spacing = 15;
-double           x_expanded_offset = 0.21;
-double           y_expanded_offset = 0.21;
 GConfClient      *gconf_client = NULL;
 
 #define DEFAULT_VARIATION "klondike.scm"
@@ -224,7 +217,7 @@ static void check_game_file_name (void)
 
 void new_game (gchar* file, guint *seedp)
 {
-  SCM size;
+  double width, height;
 
   /* If we're aborting an old game count it as a failure for
    * statistical purposes. */
@@ -270,19 +263,17 @@ void new_game (gchar* file, guint *seedp)
   }
 
   g_random_set_seed(seed);
-  score = 0;
-  set_score();
+  set_score(0);
   timer_reset ();
 
-  size = cscmi_start_game_lambda ();
+  cscmi_start_game_lambda(&width, &height);
   scm_c_eval_string ("(start-game)");
 
-  set_geometry (scm_num2double (SCM_CAR (size), 0, NULL),
-		scm_num2double (SCM_CADR (size), 0, NULL));
+  set_geometry(width, height);
 
   /* It is possible for some games to not have any moves right from the
    * start. If this happens we redeal. */
-  if (!SCM_NFALSEP (cscmi_game_over_lambda ())) {
+  if (!cscmi_game_over_lambda()) {
     new_game (file, NULL);
   } else {
     if (surface)
@@ -298,14 +289,21 @@ void new_game (gchar* file, guint *seedp)
 
 GtkWidget *score_value;
 
-void set_score () 
+guint score;
+
+void set_score(guint new_score) 
 {
   char b [10];
+  score = new_score;
   g_snprintf (b, sizeof (b), "%6d  ", score);
-
   gtk_label_set_text(GTK_LABEL(score_value), b);
 }
 
+guint get_score()
+{
+  return score;
+}
+
 GtkWidget *time_value;
 
 static gint timer_cb ()
@@ -393,6 +391,8 @@ static void create_sol_board ()
 		      GTK_SIGNAL_FUNC (configure_event), NULL);
   g_signal_connect (G_OBJECT (playing_area), "expose_event",
 		    GTK_SIGNAL_FUNC (expose_event), NULL);
+  /* No need for gtk's double buffering, since we have our own. */
+  gtk_widget_set_double_buffered(playing_area, FALSE);
 }
 
 void quit_app (GtkMenuItem *menuitem)
@@ -498,31 +498,35 @@ retrieve_state (GnomeClient *client)
                                        "/apps/aisleriot/game_file",
                                        "klondike.scm");
   card_style = games_gconf_get_string (gconf_client,
-				       "/apps/aisleriot/card_style",
-				       "bonded-new.png");
+				       THEME_GCONF_KEY,
+				       "bonded.svg");
 }
 
 int main (int argc, char *argv [])
 {
-  struct poptOption aisleriot_opts[] = {
-    {"variation", '\0', POPT_ARG_STRING, NULL, 0, NULL, NULL},
-    {NULL, '\0', 0, NULL, 0, NULL, NULL}
+  static const GOptionEntry aisleriot_opts[] = {
+    {"variation", 'v', 0, G_OPTION_ARG_STRING, &variation,
+     N_("Select the game to play"), N_("NAME")},
+    {NULL}
   };
   gchar * var_file;
-
-  aisleriot_opts[0].arg = &variation;
-  aisleriot_opts[0].descrip = N_("Variation on game rules");
-  aisleriot_opts[0].argDescrip = N_("NAME");
+  GOptionContext *option_context;
+  GnomeProgram *program;
 
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  gnome_program_init ("aisleriot", VERSION,
- 		      LIBGNOMEUI_MODULE, 
- 		      argc, argv,
- 		      GNOME_PARAM_POPT_TABLE, aisleriot_opts,
- 		      GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
+  option_context = g_option_context_new ("");
+  g_option_context_add_main_entries (option_context,
+                                     aisleriot_opts,
+                                     GETTEXT_PACKAGE);
+  
+  program = gnome_program_init ("aisleriot", VERSION,
+				LIBGNOMEUI_MODULE, 
+				argc, argv,
+				GNOME_PARAM_GOPTION_CONTEXT, option_context,
+				GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
   glade_init ();
 
   gconf_client = gconf_client_get_default ();
@@ -534,7 +538,7 @@ int main (int argc, char *argv [])
 			   NULL, NULL, NULL);
   load_statistics ();
   
-  gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-aisleriot.png");
+  gtk_window_set_default_icon_name ("gnome-aisleriot");
   g_signal_connect (GTK_OBJECT (gnome_master_client ()), "save_yourself",
 		      GTK_SIGNAL_FUNC (save_state), 
 		      (gpointer) g_path_get_basename(argv[0]));
@@ -558,5 +562,7 @@ int main (int argc, char *argv [])
   }
 
   scm_boot_guile (argc, argv, main_prog, NULL);
+
+  g_object_unref (program);
   return 0;
 }
