@@ -20,13 +20,15 @@
 #ifndef _MAIN_H_
 #define _MAIN_H_
 
+#include <games-scores.h>
+
 gint game_running (void);
 gint pause_game_cb (GtkAction *action, gpointer data);
-void update_score_state (void);
 void end_game (gboolean);
 gint new_game (void); 
 void set_numworms (int num);
 gint main_loop (gpointer data);
 
+extern GamesScores *highscores;
 
 #endif
