@@ -23,6 +23,9 @@
 #include <gnome.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
+#include <games-preimage.h>
+#include <games-find-file.h>
+
 #include <sys/types.h>
 #include <string.h>
 #include <stdlib.h>
@@ -32,17 +35,7 @@
 #include "graphics.h"
 #include "gbdefs.h"
 #include "gnobots.h"
-
-
-/**********************************************************************/
-/* GraphicInfo Structure Definition                                   */
-/**********************************************************************/
-
-typedef struct _GraphicInfo {
-  GString   *name;
-  GdkPixbuf *pixbuf;
-  GdkColor   bgcolor;
-} GraphicInfo;
+#include "properties.h"
 
 /**********************************************************************/
 
@@ -50,9 +43,13 @@ typedef struct _GraphicInfo {
 /**********************************************************************/
 /* File Static Variables                                              */
 /**********************************************************************/
-static gint          num_graphics     = -1;
-static gint          current_graphics = -1;
-static GraphicInfo **game_graphic     = NULL;
+static GamesPreimage *theme_preimage = NULL;
+static GdkPixbuf *theme_pixbuf = NULL;
+static gboolean rerender_needed = TRUE;
+
+static GdkGC *light_bggc = NULL;
+static GdkGC *dark_bggc = NULL;
+
 static GdkPixbuf    *aieee_pixbuf     = NULL;
 static GdkPixbuf    *yahoo_pixbuf     = NULL;
 static GdkPixbuf    *splat_pixbuf     = NULL;
@@ -69,6 +66,8 @@ static gint          bubble_xo   = 0;
 static gint          bubble_yo   = 0;
 static gint          bubble_type = BUBBLE_NONE;
 
+gint tile_width = 0;
+gint tile_height = 0;
 
 /**********************************************************************/
 /* Function Prototypes                                                */
@@ -84,6 +83,52 @@ static void add_bubble (gint, gint);
 /* Function Definitions                                               */
 /**********************************************************************/
 
+static void
+render_graphics (void)
+{
+  theme_pixbuf = games_preimage_render (theme_preimage,
+					14*tile_width, tile_height, 
+					NULL);
+  rerender_needed = FALSE;
+}
+
+gint resize_cb (GtkWidget *w, GdkEventConfigure *e, gpointer data)
+{
+  gint trial_width;
+  gint trial_height;
+
+  trial_width = e->width/GAME_WIDTH;
+  trial_height = e->height/GAME_HEIGHT;
+
+  if ((trial_width != tile_width) || (trial_height != tile_height)) {
+    tile_width = trial_width;
+    tile_height = trial_height;
+    rerender_needed = TRUE;
+  }
+
+  return FALSE;
+}
+
+gint expose_cb (GtkWidget *w, GdkEventExpose *e, gpointer data)
+{
+  int i, j;
+  int x1, y1, x2, y2;
+
+  x1 = e->area.x/tile_width;
+  y1 = e->area.y/tile_height;
+  x2 = x1 + e->area.width/tile_width + 1;
+  y2 = y1 + e->area.height/tile_height + 1;
+
+  for (j=y1; j<=y2; j++) {
+    for (i=x1; i<=x2; i++) {
+      /* Draw a blank space. Animation fills the objects in. */
+      draw_tile_pixmap (-1, i, j, w);
+    }
+  }
+
+  return TRUE;
+}
+
 /**
  * load_bubble_graphic
  * @fname: pixbuf filename
@@ -156,92 +201,28 @@ load_bubble_graphics (void)
 gboolean
 load_game_graphics (void)
 {
-  gint           i;
-  G_CONST_RETURN gchar* dent;
-  GDir           *dir;
-  gchar          *buffer;
-  gchar          *bptr;
-  GdkPixbuf      *image;
+  gchar          *filename;
 
-  gchar *dname = gnome_program_locate_file (NULL, 
-                                            GNOME_FILE_DOMAIN_APP_PIXMAP,
-                                            GAME_NAME, FALSE, 
-                                            NULL);
-
-  if (game_graphic != NULL) {
+  if (theme_preimage != NULL) {
     free_game_graphics ();
   }
 
-  dir = g_dir_open (dname, 0, NULL);
-  if (dir == NULL)
-    return FALSE;
+  filename = games_find_similar_file (properties_theme_name (), PIXMAPDIR);
 
-  num_graphics = 0;
-  while ((dent = g_dir_read_name (dir)) != NULL) {
-    if (! g_strrstr (dent, ".png")) {
-      continue;
-    }
-    num_graphics++;
+  theme_preimage = games_preimage_new_from_file (filename, NULL);
+  g_free (filename);
+
+  if (theme_preimage == NULL) {
+    filename = games_find_similar_file ("robots", PIXMAPDIR);
+    theme_preimage = games_preimage_new_from_file (filename, NULL);
+    g_free (filename);
   }
 
-  game_graphic = g_new (GraphicInfo*, num_graphics);
-  for (i = 0; i < num_graphics; ++i) {
-    game_graphic[i] = NULL;
-  }
-
-  g_dir_rewind (dir);
-
-  num_graphics = 0;
-  while ((dent = g_dir_read_name (dir)) != NULL) {
-    if (! g_strrstr (dent, ".png")) {
-      continue;
-    }
-    if (! strcmp (dent, "yahoo.png")) {
-      continue;
-    }
-    if (! strcmp (dent, "aieee.png")) {
-      continue;
-    }
-    if (! strcmp (dent, "splat.png")) {
-      continue;
-    }
-    if (! strcmp (dent, "gnome-gnobots2.png")) {
-      continue;
-    }
-
-    buffer = g_strdup (dent);
-    bptr = g_strrstr (buffer, ".png");
-    if (bptr != NULL)
-      *bptr = 0;
-
-    game_graphic[num_graphics] = g_new (GraphicInfo, 1);
-    game_graphic[num_graphics]->name = g_string_new (buffer);
-    g_free (buffer);
-
-    buffer = g_build_filename (dname, dent, NULL);
-
-    image = gdk_pixbuf_new_from_file (buffer, NULL);
-    g_free (buffer);
-
-    if (image == NULL) {
-      g_free (game_graphic[num_graphics]->name);
-      g_free (game_graphic[num_graphics]);
-    } else {
-      game_graphic[num_graphics]->pixbuf = image;
-      num_graphics++;
-    }
-  }
-
-  if (num_graphics == 0)
-    return FALSE;
-
-  g_dir_close (dir);
-
-  current_graphics = 0;
-
   if (! load_bubble_graphics ())
     return FALSE;
 
+  rerender_needed = TRUE;
+
   return TRUE;
 }
 
@@ -258,20 +239,15 @@ load_game_graphics (void)
 gboolean
 free_game_graphics (void)
 {
-  gint i;
-
-  if (game_graphic == NULL) {
-    return FALSE;
+  if (theme_preimage != NULL) {
+    g_object_unref (theme_preimage);
+    theme_preimage = NULL;
   }
 
-  for (i = 0; i < num_graphics; ++i) {
-    g_free (game_graphic[i]);
+  if (theme_pixbuf != NULL) {
+    g_object_unref (theme_pixbuf);
+    theme_pixbuf = NULL;
   }
-  g_free (game_graphic);
-
-  game_graphic = NULL;
-  num_graphics = -1;
-  current_graphics = -1;
 
   if (aieee_pixbuf) 
     g_object_unref (aieee_pixbuf);
@@ -288,88 +264,6 @@ free_game_graphics (void)
   return TRUE;
 }
 
-
-/**
- * num_game_graphics
- *
- * Description:
- * Returns the number of different graphics scenarios available
- *
- * Returns:
- * number of graphic types
- **/
-gint
-num_game_graphics (void)
-{
-  if (game_graphic == NULL)
-    return -1;
-
-  return num_graphics;
-}
-
-
-/**
- * ganme_graphics_name
- * @n: game graphics number
- *
- * Description:
- * The descriptive name of game graphics number @n
- *
- * Returns:
- * a string containing the graphics name
- **/
-gchar*
-game_graphics_name (gint n)
-{
-  if (game_graphic == NULL) 
-    return NULL;
-
-  if ((n < 0) || (n >= num_graphics))
-    return NULL;
-
-  return game_graphic[n]->name->str;
-}
-
-
-/**
- * game_graphics_background
- * @n: game graphics number
- *
- * Description:
- * Returns the background colour for game graphics specified by @n
- *
- * Returns:
- * background colour
- **/
-GdkColor
-game_graphics_background (gint n)
-{
-  static GdkColor nocol = {0, 0, 0, 0};
-
-  if (game_graphic == NULL) return nocol;
-
-  if ((n < 0) || (n >= num_graphics)) return nocol;
-
-  return game_graphic[n]->bgcolor;
-}
-
-
-/**
- * current_game_graphics
- *
- * Description:
- * returns the currently selected graphics
- *
- * Returns:
- * game graphics number
- **/
-gint
-current_game_graphics (void)
-{
-  return current_graphics;
-}
-
-
 /**
  * set_game_graphics
  * @ng: Game graphics number
@@ -377,30 +271,55 @@ current_game_graphics (void)
  * Description:
  * Sets the game graphics to use
  **/
-gint
-set_game_graphics (gint ng)
+void
+set_game_graphics (gchar *name)
 {
-  if ((ng < 0) || (ng >= num_graphics)) return -1;
-
-  current_graphics = ng;
-
-  if (game_area != NULL) {
-    /*set_background_color (&game_graphic[current_graphics]->bgcolor);*/
-  }
-  return current_graphics;
+  load_game_graphics ();
 }
 
 void
 set_background_color (GdkColor color)
 {
   GdkColormap *colormap;
+  guint32 brightness;
+  GdkColor color2;
 
-  if (game_area != NULL) {
-    colormap = gtk_widget_get_colormap (game_area);
-    gdk_colormap_alloc_color (colormap, &color, FALSE, TRUE);
+  if (game_area == NULL)
+    return;
 
-    gtk_widget_modify_bg (game_area, GTK_STATE_NORMAL, &color);
+  if (dark_bggc == NULL) {
+    dark_bggc = gdk_gc_new (game_area->window);
+    gdk_gc_copy (dark_bggc, game_area->style->black_gc);
+    light_bggc = gdk_gc_new (game_area->window);
+    gdk_gc_copy (light_bggc, game_area->style->white_gc);
   }
+
+  /* While the two colours are labelled "light" and "dark" which one is
+   * which actually depends on how light or dark the base colour is. */
+
+  brightness = color.red + color.green + color.blue;
+  if (brightness > 0xe8ba) { /* 0xe8ba = 0x10000/1.1 */
+    /* Darken light colours. */
+    color2.red = 0.9*color.red;
+    color2.green = 0.9*color.green;
+    color2.blue = 0.9*color.blue;
+  } else if (brightness > 0xa00) { /* Lighten darker colours. */
+    color2.red = 1.1*color.red;
+    color2.green = 1.1*color.green;
+    color2.blue = 1.1*color.blue;
+  } else { /* Very dark colours, add ratehr than multiply. */
+    color2.red += 0xa00;
+    color2.green += 0xa00;
+    color2.blue += 0xa00;
+  }
+
+  colormap = gtk_widget_get_colormap (game_area);
+  gdk_colormap_alloc_color (colormap, &color, FALSE, TRUE);
+  gdk_gc_set_foreground (dark_bggc, &color);
+  gdk_colormap_alloc_color (colormap, &color2, FALSE, TRUE);
+  gdk_gc_set_foreground (light_bggc, &color2);
+
+  clear_game_area ();
 }
 
 void
@@ -421,29 +340,42 @@ set_background_color_from_name (gchar *name)
  * draw_tile_pixmap
  * @tileno: Graphics tile number
  * @pno: Number of graphics set
- * @x: x position in pixels
- * @y: y position in pixels
+ * @x: x position in grid squares
+ * @y: y position in grid squares
  * @area: Pointer to drawing area widget
  *
  * Description:
- * Draws tile pixmap @tileno form graphics set @pno at (@x, @y) in
+ * Draws tile pixmap @tileno from graphics set @pno at (@x, @y) in
  * a widget @area
  **/
 void
-draw_tile_pixmap (gint tileno, gint pno, gint x, gint y, GtkWidget *area)
+draw_tile_pixmap (gint tileno, gint x, gint y, GtkWidget *area)
 {
-  gdk_window_clear_area (area->window, x, y, TILE_WIDTH, TILE_HEIGHT);
+  GdkGC *bg;
+
+  if ((x & 1) ^ (y & 1)) {
+    bg = dark_bggc;
+  } else {
+    bg = light_bggc;
+  }
+
+  x *= tile_width;
+  y *= tile_height;
+
+  gdk_draw_rectangle (area->window, bg, TRUE, x, y, tile_width, tile_height);
+
+  if (rerender_needed)
+    render_graphics ();
 
   if ((tileno < 0) || (tileno >= SCENARIO_PIXMAP_WIDTH)) {
     /* nothing */
   } else {
     gdk_draw_pixbuf (area->window, area->style->black_gc,
-                     game_graphic[pno]->pixbuf,
-                     tileno * TILE_WIDTH, 0,
-                     x, y, TILE_WIDTH, TILE_HEIGHT,
+                     theme_pixbuf,
+                     tileno * tile_width, 0,
+                     x, y, tile_width, tile_height,
                      GDK_RGB_DITHER_NORMAL, 0, 0);
   }
-
 }
 
 
@@ -459,30 +391,27 @@ draw_tile_pixmap (gint tileno, gint pno, gint x, gint y, GtkWidget *area)
 void
 draw_object (gint x, gint y, gint type)
 {
-  gint xpos = x * TILE_WIDTH;
-  gint ypos = y * TILE_HEIGHT;
-
-  if ((game_area == NULL) || (game_graphic == NULL)) return;
+  if (game_area == NULL) return;
 
   switch (type) {
   case OBJECT_PLAYER:
     draw_tile_pixmap (SCENARIO_PLAYER_START+player_animation, 
-                      current_graphics, xpos, ypos, game_area);
+                      x, y, game_area);
     break;
   case OBJECT_ROBOT1:
     draw_tile_pixmap (SCENARIO_ROBOT1_START+robot_animation, 
-                      current_graphics, xpos, ypos, game_area);
+                      x, y, game_area);
     break;
   case OBJECT_ROBOT2:
     draw_tile_pixmap (SCENARIO_ROBOT2_START+robot_animation, 
-                      current_graphics, xpos, ypos, game_area);
+                      x, y, game_area);
     break;
   case OBJECT_HEAP:
     draw_tile_pixmap (SCENARIO_HEAP_POS, 
-                      current_graphics, xpos, ypos, game_area);
+                      x, y, game_area);
     break;
   case OBJECT_NONE:
-    draw_tile_pixmap (-1, current_graphics, xpos, ypos, game_area);
+    draw_tile_pixmap (-1, x, y, game_area);
     break;
   }
 }
@@ -497,10 +426,9 @@ draw_object (gint x, gint y, gint type)
 void
 clear_game_area (void)
 {
-  if ((game_area == NULL) || (game_graphic == NULL)) return;
+  if (game_area == NULL) return;
   
-  gdk_window_clear_area (game_area->window, 0, 0, 
-                         GAME_WIDTH*TILE_WIDTH, GAME_HEIGHT*TILE_HEIGHT);
+  gtk_widget_queue_draw (game_area);
 }
 
 
@@ -513,7 +441,7 @@ clear_game_area (void)
 static void
 clear_bubble_area (void)
 {
-  if ((game_area == NULL) || (game_graphic == NULL)) return;
+  if (game_area == NULL) return;
   
   gdk_window_clear_area (game_area->window, bubble_xpos, bubble_ypos, 
                          BUBBLE_WIDTH, BUBBLE_HEIGHT);
@@ -627,8 +555,8 @@ draw_bubble (void)
 static void
 add_bubble (gint x, gint y)
 {
-  bubble_xpos = x * TILE_WIDTH - BUBBLE_WIDTH + BUBBLE_XOFFSET;
-  bubble_ypos = y * TILE_HEIGHT - BUBBLE_HEIGHT + BUBBLE_YOFFSET;
+  bubble_xpos = x * tile_width - BUBBLE_WIDTH + BUBBLE_XOFFSET;
+  bubble_ypos = y * tile_height - BUBBLE_HEIGHT + BUBBLE_YOFFSET;
 
   bubble_xo = 0;
   bubble_yo = 0;
