@@ -1,7 +1,7 @@
 /*
  * clock.c: 
  *
- * Copyright (C) 2001 Iain Holmes
+ * Copyright (C) 2001, 2003 Iain Holmes
  *           (C) 2001 Mark McLoughlin
  *
  * Authors: Iain Holmes <iain@ximian.com>
@@ -154,3 +154,21 @@ games_clock_set_seconds (GamesClock *clock,
 	clock->seconds = seconds;
 	clock_paint (clock);
 }
+
+time_t
+games_clock_get_seconds (GamesClock * clock)
+{
+  g_return_val_if_fail (clock && GAMES_IS_CLOCK (clock), 0);
+
+  return clock->seconds;
+}
+
+void
+games_clock_add_seconds (GamesClock *clock,
+			 time_t      seconds)
+{
+	g_return_if_fail (clock && GAMES_IS_CLOCK (clock));
+
+	clock->seconds += seconds;
+	clock_paint (clock);
+}
